use api::{Item, Transaction};
use dioxus::prelude::*;
use dioxus_material_icons::{MaterialIcon, MaterialIconStylesheet};
use dioxus_sdk::storage::use_persistent;
use qrscan::QRCodeScanPage;
use setup::{Credentials, SetupPage};

pub mod api;
pub mod page;
pub mod qrscan;
pub mod setup;

use page::{supply::SupplyPageParam, *};

#[derive(Debug, Clone, Routable, PartialEq)]
#[rustfmt::skip]
enum Route {
    #[layout(Navbar)]
    #[route("/")]
    Home {},
    #[route("/items")]
    ItemPage {},
    #[route("/flows")]
    FlowPage {},
    #[route("/locations")]
    LocationsPage {},
    #[route("/location/:location")]
    LocationPage { location: String },

    #[route("/item/:id")]
    ItemDetailPage { id: String },
    #[route("/transaction/:id")]
    TransactionPage { id: String },

    #[end_layout]
    #[route("/item/:item/:variant/consume/:id")]
    ConsumePage { id: String, item: String, variant: String },

    #[route("/item/:item/supply?:param")]
    SupplyPage { item: String, param: SupplyPageParam }
//    #[route("/blog/:id")]
//    Blog { id: i32 },

    // Pages
    // 
}

const FAVICON: Asset = asset!("/assets/favicon.ico");
const MAIN_CSS: Asset = asset!("/assets/main.css");
const TAILWIND_CSS: Asset = asset!("/assets/tailwind.css");

pub static API: GlobalSignal<Option<api::API>> = Signal::global(|| None);

fn main() {
    dioxus_sdk::set_dir!();
    dioxus::launch(App);
}

pub fn try_recover_api() -> crate::api::API {
    api::API::try_recover()
}

pub async fn fetch_api() {
    let res = use_persistent("creds", || Credentials::default())
        .read()
        .clone();
    if !res.empty() {
        let api = api::API::new(res.instance_url.clone()).await;
        *crate::API.write() = Some(api);
    }
}

#[component]
fn App() -> Element {
    let creds = use_persistent("creds", || Credentials::default());

    spawn(async move {
        fetch_api().await;
    });

    rsx! {
        document::Link { rel: "icon", href: FAVICON }
        document::Link { rel: "stylesheet", href: MAIN_CSS }
        document::Link { rel: "stylesheet", href: TAILWIND_CSS }
        MaterialIconStylesheet { }

        if creds.read().empty() {
            SetupPage {  }
        } else {
            Router::<Route> {}
        }
    }
}

#[component]
fn ItemTile(item: Item) -> Element {
    rsx! {
        Link {
            to: Route::ItemDetailPage { id: item.uuid },
            class: "flex gap-2",
            if let Some(img) = item.image {
                img { src: crate::API.read().as_ref().unwrap().get_url_instance(img.to_string()), width: "48" }
            } else {
                MaterialIcon {
                    name: "view_in_ar",
                size: 48
            }
        }
        p {
            class: "text-lg my-auto",
            {item.name.as_str()}
        }
    }
    }
}

#[component]
fn FlowPage() -> Element {
    rsx! {}
}

/// Home page
#[component]
fn QRTest() -> Element {
    let found = use_signal(|| String::new());
    let mut showQR = use_signal(|| false);

    rsx! {

        if !found.read().is_empty() {
            p {
                {format!("FOUND {}", found.read())}
            }
        },

        input {
            type: "text"
        }

        if *showQR.read() {
            QRCodeScanPage { result: found, show: showQR }
        },

                button {
            onclick: move |_| {
                showQR.set(true);
            },
            "Click"
        }

    }
}

#[component]
fn TransactionCard(t: Transaction) -> Element {
    rsx! {
    // TODO : transaction card
    Link {
        to: Route::TransactionPage { id: t.uuid.clone() },
        p { {t.uuid.as_str()} }
    }
        }
}
#[component]
fn Navbar() -> Element {
    rsx! {
        div {
            id: "navbar",
            class: "flex flex-wrap gap-2 p-2 shadow-md rounded-sm",
            style: "background-color:hsl(223, 18.90%, 13.30%);",
            Link {
                to: Route::Home {},
                class: "flex items-center gap-2 px-3 py-2 rounded hover:bg-gray-100 dark:hover:bg-gray-800 cursor-pointer text-gray-900 dark:text-gray-100",
                MaterialIcon { name: "home", size: 24 },
                "Home"
            }
            Link {
                to: Route::ItemPage {  },
                class: "flex items-center gap-2 px-3 py-2 rounded hover:bg-gray-100 dark:hover:bg-gray-800 cursor-pointer text-gray-900 dark:text-gray-100",
                MaterialIcon { name: "category", size: 24 },
                "Items"
            }
            Link {
                to: Route::FlowPage {  },
                class: "flex items-center gap-2 px-3 py-2 rounded hover:bg-gray-100 dark:hover:bg-gray-800 cursor-pointer text-gray-900 dark:text-gray-100",
                MaterialIcon { name: "assignment", size: 24 },
                "Flows"
            }
            Link {
                to: Route::LocationsPage {  },
                class: "flex items-center gap-2 px-3 py-2 rounded hover:bg-gray-100 dark:hover:bg-gray-800 cursor-pointer text-gray-900 dark:text-gray-100",
                MaterialIcon { name: "map", size: 24 },
                "Locations"
            }
        }

        div {
            class: "py-4",
            Outlet::<Route> {}
        }
    }
}
